import { GetServerSideProps } from 'next';
import Link from 'next/link';
import { apiClientGet } from '@/lib/apiClient';
import { gql } from 'apollo-server-micro';
import { GameLight, GameLightFragment } from '@/lib/game';
import Error500, { Error500Props } from '../500';
import { extendFragment, includeFragment } from '@/lib/fragment';
import { Paginated, paginationQuery } from '@/lib/page';

type Game = {
  id: string;
  name: string;
  system: string;
};

type PageProps = {
  games:Paginated<GameLight>|null;
} | Error500Props;

export const getServerSideProps: GetServerSideProps<PageProps> = async () => {
  try {
    const client = await apiClientGet();
    const res = await client.query<{ games:Paginated<GameLight> }>({
      variables: { },
      query: gql`
        ${includeFragment(GameLightFragment)}
        query getGames {
          games(first: 100) {
            ${paginationQuery(extendFragment(GameLightFragment))}
          }
        }
      `,
    });

    if(!res || !res.data || !res.data.games) {
      return { props: { games:null } };
    }

    return { props: { games: res.data.games  } };
  } catch (e) {
    console.error('Error', e);
    return { props: { code: 500 } };
  }
};

export const Page: React.FC<PageProps> = props => {
  if ('code' in props) {
    if (props.code === 500) return <Error500 {...props} />;
    return null;
  }

  const { games } = props;
  if(!games) {
    return <div>No games found</div>;
  }

  return (
    <div>
      <h1>Games</h1>
      <ul>
        {games.edges.map(({ node }) => {
          return (
            <li key={node.id}>
              <Link href={`/games/${node.id}`}>
                {node.name} ({node.system})
              </Link>
            </li>
          );
        })}
      </ul>
    </div>
  );
};

export default Page;